import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;
import org.jdom.JDOMException;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.StringUtils;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import java.util.ArrayList;
import org.hibernate.HibernateException;
import org.hibernate.SQLQuery;
import com.ephesoft.dcma.script.IJDomScript;
import com.ephesoft.dcma.core.DCMAException;
import com.ephesoft.dcma.core.hibernate.DynamicHibernateDao;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;

public class ScriptAutomaticValidation implements IJDomScript
{
	private static final String BATCH_LOCAL_PATH = "BatchLocalPath";
	private static final String BATCH_INSTANCE_ID = "BatchInstanceIdentifier";
	private static final String EXT_BATCH_XML_FILE = "_batch.xml";
	private static String ZIP_FILE_EXT = ".zip";
	private static Properties INVOICE_PROPERTIES = null;
	private static final String DOC_TYPE = "INVOICE";
	private static final String FIELD_TO_VALIDATE = "PO_Number";
		
	public Object execute(Document documentFile, String methodName, String docIdentifier)
	{
		Exception exception = null;
		try
		{
			System.out.println("*************  Inside ScriptAutomaticValidation scripts.");
			System.out.println("*************  Start execution of the AutomaticValidation scripts.");
			if (null == documentFile)
			{
				System.out.println("Input document is null.");
			}
			
			loadpropertyFile(documentFile);
			if (getPropertyValue("TableLineItemSummation").equalsIgnoreCase("true"))
			{
				//checkTable(documentFile);
			}
			if (getPropertyValue("PONumberDBValidationSwitch").equalsIgnoreCase("true"))
			{
				//validatePO(documentFile);
			}
			
			System.out.println("*************  Successfully write the xml file for the AutomaticValidation scripts.");
			writeToXML(documentFile);
			System.out.println("*************  End execution of the AutomaticValidation scripts.");
		}
		catch (Exception e)
		{
			System.out.println("*************  Error occurred in scripts." + e.getMessage());
			exception = e;
		}
		return null;
	}

	public static void loadpropertyFile(Document document)
	{
		Element docRoot = document.getRootElement();
		String batchClassId = docRoot.getChildText("BatchClassIdentifier");
		String batchLocalPath = docRoot.getChildText("BatchLocalPath");
		String propertiesFilePath = batchLocalPath.substring(0, (batchLocalPath.length() - 22)) + batchClassId + "\\script-config\\InvoiceBatchClass.properties";
		Properties prop = new Properties();
		InputStream input = null;
		try
		{
			input = new FileInputStream(propertiesFilePath);
			prop.load(input);
			INVOICE_PROPERTIES = prop;
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	public static String getPropertyValue(String propertykey)
	{
		String propertyValue = INVOICE_PROPERTIES.getProperty(propertykey);
		return propertyValue;
	}

	private void validatePO(Document document)
	{
		String SQL_USER = getPropertyValue("UserName");
		String SQL_PASSWORD = getPropertyValue("Password");
		String SQL_DRIVER = null;
		String SERVER_NAME = getPropertyValue("ServerName");;
		String DATABASE_NAME = getPropertyValue("DatabaseName");;
		String DATABASE_PORT = getPropertyValue("DatabasePort");
		String SQL_CONNECTION_STRING = null;
		String SQL_TABLE = getPropertyValue("LookupTableName");
		String COLUMN = getPropertyValue("LookupColumn");
		Map<String,String> p = new HashMap<String,String>();
		
		if (getPropertyValue("DBConnectionType").trim().equalsIgnoreCase(("MYSQL")))
		{
			SQL_DRIVER = "com.mysql.jdbc.Driver";
			SQL_CONNECTION_STRING = "jdbc:mysql://" + SERVER_NAME + ":" + DATABASE_PORT + "/" + DATABASE_NAME;
			System.out.println(SQL_CONNECTION_STRING);
		}
		else if (getPropertyValue("DBConnectionType").trim().equalsIgnoreCase(("MSSQL")))
		{
			SQL_DRIVER = "net.sourceforge.jtds.jdbc.Driver";
			SQL_CONNECTION_STRING = "jdbc:jtds:sqlserver://" + SERVER_NAME;
			if (DATABASE_PORT != null)
			{
				if (!DATABASE_PORT.isEmpty()) SQL_CONNECTION_STRING += ":" + DATABASE_PORT;
			}
			SQL_CONNECTION_STRING += "/" + DATABASE_NAME;
		}
		
		DynamicHibernateDao dynamicHibernateDao = null;
		Element docRoot = document.getRootElement();
		
		try
		{
			dynamicHibernateDao = new DynamicHibernateDao(SQL_USER, SQL_PASSWORD, SQL_DRIVER, SQL_CONNECTION_STRING);
			List<Element> docList = docRoot.getChild("Documents").getChildren("Document");
			for (Element doc : docList)
			{
				StringBuffer dbQueryBuffer = new StringBuffer();
				String queryString = "";
				String docId = doc.getChildText("Identifier");
				String docType = doc.getChildText("Type");
				Element errorElement = (Element) doc.getChild("ErrorMessage");
				if (docType.equals(DOC_TYPE))
				{
					List<Element> dlfList = doc.getChild("DocumentLevelFields").getChildren("DocumentLevelField");
					for (Element dlf : dlfList)
					{
						String dlfName = dlf.getChildText("Name");
						String dlfValue = dlf.getChildText("Value");
					
						if (dlfName.equals(FIELD_TO_VALIDATE))
						{
							if (dlfValue != null)
							{
								if (!dlfValue.equals(""))
								{				
									String reformattedPONum = dlfValue;
									if (SQL_CONNECTION_STRING.contains("mysql"))
									{
										queryString = "SELECT COUNT(*) FROM " + SQL_TABLE + " WHERE " + COLUMN + " = '" + reformattedPONum + "';";
									}
									else if (SQL_CONNECTION_STRING.contains("sqlserver"))
									{
										queryString = "SELECT COUNT_BIG(*) FROM " + SQL_TABLE + " WHERE " + COLUMN + " = '" + reformattedPONum + "';";
									}
									
									SQLQuery query = dynamicHibernateDao.createQuery(queryString);
									List<Object> result = new ArrayList<Object>();
									result = query.list();
									System.out.println(result.get(0));
									if (result.get(0).equals(BigInteger.ZERO))
									{
										errorElement.setText("PO Number " + dlfValue + " not found in database.");
									}
								}
							}
						}
					}
				}
			}
			dynamicHibernateDao.closeSession();
		}
		catch (HibernateException e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		catch (DCMAException x)
		{
			x.printStackTrace();
		}
		finally
		{
			if (dynamicHibernateDao != null)
			{
				dynamicHibernateDao.closeSession();
			}
		}
	}

	public void checkTable(Document documentfile)
	{
		String COLUMN = getPropertyValue("TableLineItemExtendedPrice");
		try
		{
			XPath documentsPath = XPath.newInstance("/Batch/Documents/Document[Type='INVOICE']");
			List<Element> documents = documentsPath.selectNodes(documentfile);
			for (Element doc : documents)
			{
				Boolean validInput = true;
				XPath valuePath = XPath.newInstance("DataTables/DataTable[Name='INVOICE TABLE']/Rows/Row/Columns/Column[Name='" + COLUMN + "']/Value");
				List<Element> hoursList = valuePath.selectNodes(doc);
				BigDecimal totalHours = new BigDecimal("0.00");
				for (Element hoursElement : hoursList)
				{
					String hourString = hoursElement.getText();
					hourString = StringUtils.remove(hourString, ',');
					hourString = StringUtils.remove(hourString, '$');
					Boolean validHours = NumberUtils.isNumber(hourString);
					if (validHours)
					{
						BigDecimal tempHour = new BigDecimal(hourString);
						totalHours = totalHours.add(tempHour);
						log("Extended Amount: " + hoursElement.getText());
					}
					else
					{
						log("Invalid Input: " + hoursElement.getText());
						validInput = false;
					}
				}
				XPath subtotal = XPath.newInstance("DocumentLevelFields/DocumentLevelField[Name='" + getPropertyValue("TableLineItemSummationCompareField") + "']");
				Element subTotalElement = (Element) subtotal.selectSingleNode(doc);
				log("	Extended Sum: " + totalHours.doubleValue());
				String subTotalString = subTotalElement.getChildText("Value");
				subTotalString = StringUtils.remove(subTotalString, ',');
				subTotalString = StringUtils.remove(subTotalString, '$');
				log("	Subtotal value: " + subTotalElement.getChildText("Value"));
				Boolean validSub = NumberUtils.isNumber(subTotalString);
				if (validSub && validInput)
				{
					double subtotalValue = Double.parseDouble(subTotalString);
					if (subtotalValue == totalHours.doubleValue())
					{
						log("	SubTotal is equal to Extended Amount sum");
					}
					else
					{
						log("	SubTotal is not equals to the Extended Amount sum");
						setErrorMessage(doc);
					}
				}
				else
				{
					if (!validInput) log("An extended amount field in the table is invalid");
					if (!validSub) log("subTotal field is invalid: " + subTotalElement.getChildText("Value"));
					setErrorMessage(doc);
				}
			}
			writeToXML(documentfile);
		}
		catch (JDOMException e)
		{
			System.err.println("script failed: " + e);
			e.printStackTrace();
		}
	}
	
	public void setErrorMessage(Element document)
	{
		log(document.getChildText("Identifier"));
		Element errorElement = document.getChild("ErrorMessage");
		errorElement.setText("SubTotal is not equal to the extended amount table sum");
	}

	private void log(String log)
	{
		System.out.println(log);
	}

	private void writeToXML(Document document)
	{
		String batchLocalPath = null;
		List<?> batchLocalPathList = document.getRootElement().getChildren(BATCH_LOCAL_PATH);
		if (null != batchLocalPathList)
		{
			batchLocalPath = ((Element) batchLocalPathList.get(0)).getText();
		}
		if (null == batchLocalPath)
		{
			System.err.println("Unable to find the local folder path in batch xml file.");
			return;
		}
		String batchInstanceID = null;
		List<?> batchInstanceIDList = document.getRootElement().getChildren(BATCH_INSTANCE_ID);
		if (null != batchInstanceIDList)
		{
			batchInstanceID = ((Element) batchInstanceIDList.get(0)).getText();
		}
		if (null == batchInstanceID)
		{
			System.err.println("Unable to find the batch instance ID in batch xml file.");
			return;
		}
		String batchXMLPath = batchLocalPath.trim() + File.separator + batchInstanceID + File.separator + batchInstanceID + EXT_BATCH_XML_FILE;
		String batchXMLZipPath = batchXMLPath + ZIP_FILE_EXT;
		System.out.println("batchXMLZipPath************" + batchXMLZipPath);
		OutputStream outputStream = null;
		File zipFile = new File(batchXMLZipPath);
		FileWriter writer = null;
		XMLOutputter out = new XMLOutputter();
		try
		{
			if (zipFile.exists())
			{
				System.out.println("Found the batch xml zip file.");
				outputStream = getOutputStreamFromZip(batchXMLPath, batchInstanceID + EXT_BATCH_XML_FILE);
				out.output(document, outputStream);
			}
			else
			{
				writer = new java.io.FileWriter(batchXMLPath);
				out.output(document, writer);
				writer.flush();
				writer.close();
			}
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
		}
		finally
		{
			if (outputStream != null)
			{
				try
				{
					outputStream.close();
				}
				catch (IOException e)
				{
				}
			}
		}
	}

	public static OutputStream getOutputStreamFromZip(final String zipName, final String fileName) throws FileNotFoundException, IOException
	{
		ZipOutputStream stream = null;
		stream = new ZipOutputStream(new FileOutputStream(new File(zipName + ZIP_FILE_EXT)));
		ZipEntry zipEntry = new ZipEntry(fileName);
		stream.putNextEntry(zipEntry);
		return stream;
	}

	public static void main(String args[])
	{
		String filePath = "C:\\Ephesoft\\SharedFolders\\ephesoft-system-folder\\BIE\\scriptTest\\BIE_batch.xml";
		try
		{
			SAXBuilder sb = new SAXBuilder();
			Document doc = sb.build(filePath);
			ScriptAutomaticValidation se = new ScriptAutomaticValidation();
			se.execute(doc, null, null);
		}
		catch (Exception x)
		{
			System.out.println(x);
		}
	}
}
