import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.hibernate.HibernateException;
import org.hibernate.SQLQuery;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import com.ephesoft.dcma.core.DCMAException;
import com.ephesoft.dcma.core.hibernate.DynamicHibernateDao;
import com.ephesoft.dcma.script.IJDomScript;

public class ScriptFieldValueChange implements IJDomScript
{
	private static String BATCH_LOCAL_PATH = "BatchLocalPath";
	private static String BATCH_INSTANCE_ID = "BatchInstanceIdentifier";
	private static String ZIP_FILE_EXT = ".zip";
	private static String EXT_BATCH_XML_FILE = "_batch.xml";	
	private static String FIELD_TO_VALIDATE = "PO_Number";
	private static Properties INVOICE_PROPERTIES = null;

	public Object execute(Document document, String fieldName, String docIdentifier)
	{
		System.out.println("*************  Inside field value change script.");
		System.out.println("*************  Start execution of field value change  script.");
		Exception exception = null;
		try
		{
			if (fieldName.equals(FIELD_TO_VALIDATE))
			{
				loadpropertyFile(document);
				validatePO(document, fieldName, docIdentifier);
				writeToXML(document);
			}
		}
		catch (Exception e)
		{
			System.out.println("*************  Error occurred in scripts." + e.getMessage());
			exception = e;
			e.printStackTrace();
		}
		System.out.println("*************  End execution of the Field Value Change script.");
		return exception;
	}

	public static void loadpropertyFile(Document document)
	{
		Element docRoot = document.getRootElement();
		String batchClassId = docRoot.getChildText("BatchClassIdentifier");
		String batchLocalPath = docRoot.getChildText("BatchLocalPath");
		String propertiesFilePath = batchLocalPath.substring(0, (batchLocalPath.length() - 22)) + batchClassId + "\\script-config\\InvoiceBatchClass.properties";
		Properties prop = new Properties();
		InputStream input = null;
		try
		{
			input = new FileInputStream(propertiesFilePath);
			prop.load(input);
			INVOICE_PROPERTIES = prop;
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			if (input != null)
			{
				try
				{
					input.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	public static String getPropertyValue(String propertykey)
	{
		String propertyValue = INVOICE_PROPERTIES.getProperty(propertykey);
		return propertyValue;
	}

	private void validatePO(Document document, String fieldName, String docIdentifier)
	{
		String SQL_USER = getPropertyValue("UserName");
		String SQL_PASSWORD = getPropertyValue("Password");
		String SQL_DRIVER = null;
		String SERVER_NAME = getPropertyValue("ServerName");;
		String DATABASE_NAME = getPropertyValue("DatabaseName");;
		String DATABASE_PORT = getPropertyValue("DatabasePort");
		String SQL_CONNECTION_STRING = null;
		String SQL_TABLE = getPropertyValue("LookupTableName");
		String COLUMN = getPropertyValue("LookupColumn");
		String PONUMBER_REGEX = getPropertyValue("PONumberRegex");
		
		if (getPropertyValue("DBConnectionType").trim().equalsIgnoreCase(("MYSQL")))
		{
			SQL_DRIVER = "com.mysql.jdbc.Driver";
			SQL_CONNECTION_STRING = "jdbc:mysql://" + SERVER_NAME + ":" + DATABASE_PORT + "/" + DATABASE_NAME;
		}
		else if (getPropertyValue("DBConnectionType").trim().equalsIgnoreCase(("MSSQL")))
		{
			SQL_DRIVER = "net.sourceforge.jtds.jdbc.Driver";
			SQL_CONNECTION_STRING = "jdbc:jtds:sqlserver://" + SERVER_NAME;
			
			if (DATABASE_PORT != null)
			{
				if (!DATABASE_PORT.isEmpty()) SQL_CONNECTION_STRING += ":" + DATABASE_PORT;
			}
		
			SQL_CONNECTION_STRING += "/" + DATABASE_NAME;
		}
		DynamicHibernateDao dynamicHibernateDao = null;
		Element docRoot = document.getRootElement();

		Pattern p = Pattern.compile(PONUMBER_REGEX);
		try
		{

			try
			{
				dynamicHibernateDao = new DynamicHibernateDao(SQL_USER, SQL_PASSWORD, SQL_DRIVER, SQL_CONNECTION_STRING);
			}
			catch (DCMAException e1)
			{
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			List<Element> docList = docRoot.getChild("Documents").getChildren("Document");
			for (Element doc : docList)
			{

				String queryString = "";
				String docId = doc.getChildText("Identifier");

				Element errorElement = (Element) doc.getChild("ErrorMessage");
				if (docId.equals(docIdentifier))
				{
					List<Element> dlfList = doc.getChild("DocumentLevelFields").getChildren("DocumentLevelField");
					for (Element dlf : dlfList)
					{
						String dlfName = dlf.getChildText("Name");
						String dlfValue = dlf.getChildText("Value");
						if (dlfName.equals(FIELD_TO_VALIDATE))
						{
							if (dlfValue != null)
							{
								if (!dlfValue.trim().equals(""))
								{
									Matcher m = p.matcher(dlfValue);
									if (m.find())
									{
										try
										{
											String reformattedPONum = dlfValue;
											if (SQL_CONNECTION_STRING.contains("mysql"))
											{
											
												queryString = "SELECT COUNT(*) FROM " + SQL_TABLE + " WHERE " + COLUMN + " = '" + reformattedPONum + "';";
											}
											else if (SQL_CONNECTION_STRING.contains("sqlserver"))
											{
												queryString = "SELECT COUNT_BIG(*) FROM " + SQL_TABLE + " WHERE " + COLUMN + " = '" + reformattedPONum + "';";
											}
											SQLQuery query = dynamicHibernateDao.createQuery(queryString);
											List<Object> result = new ArrayList<Object>();
											result = query.list();
										
											if (result.get(0).equals(BigInteger.ZERO))
											{
												errorElement.setText("PO Number " + dlfValue + " not found in database.");
											}
										}catch (Exception e){}
									}
									else 
									{
										errorElement.setText("PO Number " + dlfValue + " does not conform to PO Regex");
									}
								}
								else
								{
									
									errorElement.setText("");
								}
							}
						}
					}
				}
			}
			dynamicHibernateDao.closeSession();
		}
		catch (HibernateException e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		finally
		{
			if (dynamicHibernateDao != null)
			{
				dynamicHibernateDao.closeSession();
			}
		}
	}

	private void writeToXML(Document document)
	{
		String batchLocalPath = null;
		List<?> batchLocalPathList = document.getRootElement().getChildren(BATCH_LOCAL_PATH);
		if (null != batchLocalPathList)
		{
			batchLocalPath = ((Element) batchLocalPathList.get(0)).getText();
		}
		if (null == batchLocalPath)
		{
			System.err.println("Unable to find the local folder path in batch xml file.");
			return;
		}
		String batchInstanceID = null;
		List<?> batchInstanceIDList = document.getRootElement().getChildren(BATCH_INSTANCE_ID);
		if (null != batchInstanceIDList)
		{
			batchInstanceID = ((Element) batchInstanceIDList.get(0)).getText();
		}
		if (null == batchInstanceID)
		{
			System.err.println("Unable to find the batch instance ID in batch xml file.");
			return;
		}
		String batchXMLPath = batchLocalPath.trim() + File.separator + batchInstanceID + File.separator + batchInstanceID + EXT_BATCH_XML_FILE;
		String batchXMLZipPath = batchXMLPath + ZIP_FILE_EXT;
		System.out.println("batchXMLZipPath************" + batchXMLZipPath);
		OutputStream outputStream = null;
		File zipFile = new File(batchXMLZipPath);
		FileWriter writer = null;
		XMLOutputter out = new XMLOutputter();
		try
		{
			if (zipFile.exists())
			{
				System.out.println("Found the batch xml zip file.");
				outputStream = getOutputStreamFromZip(batchXMLPath, batchInstanceID + EXT_BATCH_XML_FILE);
				out.output(document, outputStream);
			}
			else
			{
				writer = new java.io.FileWriter(batchXMLPath);
				out.output(document, writer);
				writer.flush();
				writer.close();
			}
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
		}
		finally
		{
			if (outputStream != null)
			{
				try
				{
					outputStream.close();
				}
				catch (IOException e)
				{
				}
			}
		}
	}

	public static OutputStream getOutputStreamFromZip(final String zipName, final String fileName) throws FileNotFoundException, IOException
	{
		ZipOutputStream stream = null;
		stream = new ZipOutputStream(new FileOutputStream(new File(zipName + ZIP_FILE_EXT)));
		ZipEntry zipEntry = new ZipEntry(fileName);
		stream.putNextEntry(zipEntry);
		return stream;
	}
	
	// use the main method to test scripts outside of Ephesoft
	public static void main(String args[])
	{
		// Define a path to the Batch XML. In this case the BI19_Batch.xml was
		// extracted and placed into a folder that was created call scriptdev
		String filePath = "C:\\Ephesoft\\SharedFolders\\ephesoft-system-folder\\BIE\\scriptTest\\BIE_batch.xml";
		try
		{
			SAXBuilder sb = new SAXBuilder();
			Document doc = sb.build(filePath);
			// Note that the name of the script is being used for the execute
			ScriptFieldValueChange se = new ScriptFieldValueChange();
			se.execute(doc, "PO_Number", "DOC1");
		}
		catch (Exception x)
		{
			System.out.println("Exception");
			x.printStackTrace();
		}
	} 
}
