import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import com.ephesoft.dcma.script.IJDomScript;

public class ScriptFunctionKey implements IJDomScript
{
	private static String BATCH_LOCAL_PATH = "BatchLocalPath";
	private static String BATCH_INSTANCE_ID = "BatchInstanceIdentifier";
	private static String EXT_BATCH_XML_FILE = "_batch.xml";
	private static String ZIP_FILE_EXT = ".zip";

	public Object execute(Document document, String methodName, String docIdentifier)
	{
		Exception exception = null;
		try
		{
			Class<ScriptFunctionKey> c = ScriptFunctionKey.class;
			Method meth = c.getMethod(methodName, String.class, Document.class);
			ScriptFunctionKey scriptFunctionKey = new ScriptFunctionKey();
			meth.invoke(scriptFunctionKey, docIdentifier, document);
		}
		catch (Exception e)
		{
			System.out.println("*************  Error occurred in scripts." + e.getMessage());
			e.printStackTrace();
			exception = e;
		}
		return exception;
	}

	public void deleteVendorInfo(String docIdentifier, Document document)
	{
		String[] vendorFields = {"VendorName", "VendorNumber", "VendorAddress", "VendorPhone" };
		Element docRoot = document.getRootElement();
		List<Element> docList = docRoot.getChild("Documents").getChildren("Document");
		for (Element doc : docList)
		{
			String docId = doc.getChildText("Identifier");
			if (docId.equals(docIdentifier))
			{
				doc.getChild("Valid").setText("false");
				List<Element> dlfList = doc.getChild("DocumentLevelFields").getChildren("DocumentLevelField");
				for (Element dlf : dlfList)
				{
					Element dlfName = dlf.getChild("Name");
					Element dlfValue = dlf.getChild("Value");
					for (String s : vendorFields)
					{
						if (dlfName.getText().equals(s))
						{
							if (dlfValue != null) dlfValue.setText("");
						}
					}
				}
				break;
			}
		}
		writeToXML(document);
	}

	private void writeToXML(Document document)
	{
		String batchLocalPath = null;
		List<?> batchLocalPathList = document.getRootElement().getChildren(BATCH_LOCAL_PATH);
		if (null != batchLocalPathList)
		{
			batchLocalPath = ((Element) batchLocalPathList.get(0)).getText();
		}
		if (null == batchLocalPath)
		{
			System.err.println("Unable to find the local folder path in batch xml file.");
			return;
		}
		String batchInstanceID = null;
		List<?> batchInstanceIDList = document.getRootElement().getChildren(BATCH_INSTANCE_ID);
		if (null != batchInstanceIDList)
		{
			batchInstanceID = ((Element) batchInstanceIDList.get(0)).getText();
		}
		if (null == batchInstanceID)
		{
			System.err.println("Unable to find the batch instance ID in batch xml file.");
			return;
		}
		String batchXMLPath = batchLocalPath.trim() + File.separator + batchInstanceID + File.separator + batchInstanceID + EXT_BATCH_XML_FILE;
		String batchXMLZipPath = batchXMLPath + ZIP_FILE_EXT;
		System.out.println("batchXMLZipPath************" + batchXMLZipPath);
		OutputStream outputStream = null;
		File zipFile = new File(batchXMLZipPath);
		FileWriter writer = null;
		XMLOutputter out = new XMLOutputter();
		try
		{
			if (zipFile.exists())
			{
				System.out.println("Found the batch xml zip file.");
				outputStream = getOutputStreamFromZip(batchXMLPath, batchInstanceID + EXT_BATCH_XML_FILE);
				out.output(document, outputStream);
			}
			else
			{
				writer = new java.io.FileWriter(batchXMLPath);
				out.output(document, writer);
				writer.flush();
				writer.close();
			}
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
		}
		finally
		{
			if (outputStream != null)
			{
				try
				{
					outputStream.close();
				}
				catch (IOException e)
				{
				}
			}
		}
	}

	public static OutputStream getOutputStreamFromZip(final String zipName, final String fileName) throws FileNotFoundException, IOException
	{
		ZipOutputStream stream = null;
		stream = new ZipOutputStream(new FileOutputStream(new File(zipName + ZIP_FILE_EXT)));
		ZipEntry zipEntry = new ZipEntry(fileName);
		stream.putNextEntry(zipEntry);
		return stream;
	}
}
